%
% CheckGridTransformation.m
%

function CheckGridTransformation(f, gridData, varargin)

  aspect = GetAspect(varargin{:});

  range = GridRange(gridData);

  gridPoints = gridData.GetFlatMatrix();
  gridValues = f(gridPoints')';

  for i = 1:size(gridValues, 2)
    if ~IsValueInRange(gridValues(:, i), range)
      message = BuildLogMessage(gridPoints(:, i), gridValues(:, i));
      if aspect ~= 0
        aspect.Log(message);
      else
        disp(message);
      end
    end
  end

end

function result = GridRange(gridData)
  result = zeros(gridData.Dim, 2);
  for d = 1:gridData.Dim
    values = gridData.Grids{d};
    result(d, :) = [min(values) max(values)];
  end
end

function result = IsValueInRange(value, range)
  dim = size(value, 1);
  result = prod((range(1:dim, 1) <= value) .* (value <= range(1:dim, 2)));
end

function result = GetAspect(varargin)
  if size(varargin, 2) > 0
    result = varargin{1};
  else
    result = 0;
  end
end

function result = BuildLogMessage(point, value)
  result = sprintf('f([%s]) = ([%s]) is out of grids', ToCsv(point'), ToCsv(value'));
end

function result = ToCsv(values)
  result = sprintf('%g,', values);
  result = result(1:end-1);
end
