%
% GridData.m
%
classdef GridData < handle

  properties (SetAccess = private)
    Grids
    Dim
    CountPerDim
  end
  
  methods (Access = public)
  
    function this = GridData(grids)
      this.Grids = grids;
      if (iscell(grids))
        this.Dim = size(grids, 2);
        if (this.Dim == 1)
          this.CountPerDim(1) = size(grids{1}, 2);
        else
          m = grids{this.Dim};
          this.CountPerDim(this.Dim) = size(m, 2);
          for i = this.Dim-1:-1:1
            m = this.combMatrix(grids{i}, m);
            this.CountPerDim(i) = size(grids{i}, 2);
          end
        end
      else
        this.Dim = 1;
        this.CountPerDim(1) = size(grids, 2);
      end
    end

    %
    %
    %
    function result = GetIndices(this, pivot)
      major  = this.getMajorOffsets(pivot);
      minor  = this.getMinorOffsets(pivot);
      step   = this.getSteps(pivot);
      result = this.buildIndices(major, minor, step);
    end

    %
    %
    %
    function result = At(this, indices)
      result = zeros(this.Dim, length(indices));
      i = indices - 1;
      for d = 1:this.Dim
        result(d, :) = this.Grids{d}(mod(i, this.CountPerDim(d)) + 1);
        i = floor(i ./ this.CountPerDim(d));
      end
    end

    %
    %
    %
    function result = GetAllIndices(this)
      result = 1:prod(this.CountPerDim);
    end

    %
    %
    %
    function result = GetFlatMatrix(this)
      result = this.At(this.GetAllIndices());
    end

    %
    % 
    %
    function result = MakeSameDimData(this, data)
      % data count check
      expectedCount = 1;
      for i = 1:size(this.CountPerDim, 2)
        expectedCount = expectedCount * this.CountPerDim(i);
      end
      if (size(data, 2) ~= expectedCount)
        error('Data count must be %d.', expectedCount);
      end
      
      % logic
      result = this.makeArray(data, 1);
    end
    
  end
  
  methods (Access = private)

    function result = getMajorOffsets(this, pivot)
      if pivot == this.Dim
        result = [0];
      else
        result = [0:prod(this.CountPerDim(pivot+1:end))-1] .* prod(this.CountPerDim(1:pivot));
      end
    end

    function result = getMinorOffsets(this, pivot)
      if pivot == 1
        result = [0];
      else
        result = [0:prod(this.CountPerDim(1:pivot-1))-1];
      end
    end

    function result = getSteps(this, pivot)
      if pivot == 1
        result = [0:this.CountPerDim(pivot)-1];
      else
        result = [0:this.CountPerDim(pivot)-1] .* prod(this.CountPerDim(1:pivot-1));
      end
    end

    function result = buildIndices(this, major, minor, step)
      result = repmat(major, length(minor), 1) + repmat(minor, length(major), 1)';
      result = repmat(result(:), 1, length(step));
      result = result + repmat(step, length(result), 1) + 1;
    end

    function result = combMatrix(this, v, m)
      result = [];
      for i = 1:size(v, 2)
        result = horzcat(result, vertcat(repmat(v(i), 1, size(m, 2)), m));
      end
    end
    
    function result = makeArray(this, data, dim)
      result = [];
      s = size(data, 2);
      if (this.CountPerDim(dim) == s)
        for i = 1:s
          result = cat(dim, result, data(i));
        end
      else
        u = s / this.CountPerDim(dim);
        for i = 1:this.CountPerDim(dim)
          result = cat(dim, result, this.makeArray(data((i - 1) * u + 1:i * u), dim + 1));
        end
      end
    end
    
  end
  
end
