%
% LinearStateSpaceModelPMMHEstimation.m
%

rng('shuffle');

%
% Generate state and observation time series from Linear state space model
%

T    = 1000;
k1   = 0.1;
k2   = 0.1;
mass = 1.0;
q    = 0.1;
r    = 0.1;
dt   = 1.0;

[x, y, F, H] = LinearStateSpaceModel(T, k1, k2, mass, q, r, dt);

save('logs/LinearStateSpaceModel_Data.mat');

grids = { ...
  [-2:0.5:2], ...
  [-2:0.5:2]  ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

obsKernelGens = {
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

stateMeanFuncs = { ...
  @(x) x(1,:), ...
  @(x) x(2,:)  ...
};

obsMeanFuncs = { ...
  @(x) x(1,:)  ...
};

x0 = zeros(1, 2);
N  = 500;
J  = 100000;

aspect = Aspect();
aspect.LogFileName = 'logs/LinearStateSpaceModelPMMH.log';
aspect.MatFileNamePrefix = 'logs/LinearStateSpaceModelPMMH';
aspect.SavesIntermediateMat = true;

disp('PMMH Estimation using SPAPI');

splineHandle = @GenericSpline;
[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  PMMH(), ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  0, ... % Not used in PMMH estimation
  splineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  [], ... % No controls
  y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('logs/LinearStateSpaceModelPMMH.mat');

disp('Done');
